/*
 * $Header: /home/cvs/jakarta-slide/src/contrib/webdavgui/src/java/org/apache/webdav/ui/filechooser/SPFileView.java,v 1.2 2001/07/17 04:00:46 msmith Exp $
 * $Revision: 1.2 $
 * $Date: 2001/07/17 04:00:46 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.webdav.ui.filechooser;

import java.awt.*;
import javax.swing.*;
import javax.swing.filechooser.*;

import java.io.*;

import org.apache.webdav.ui.filechooser.dir.*;

/**
 * Title:        SPFileView.java
 * Description:  Implements javax.swing.filechooser.FileView to
 *               provide the filechooser with ui information of a File.
 * Company:      SpeedLegal Holdings Inc.
 * @author       Jojada J. Tirtowidjojo
 * @version 1.0
 */

public class SPFileView extends FileView {

  private ImageIcon docIcon, redDocIcon;
  private ImageIcon folderIcon, redFolderIcon;
  private ImageIcon unknownIcon;

  public SPFileView() {
      docIcon = new ImageIcon(getClass().
	    getResource("/images/docs.gif"));
      redDocIcon = new ImageIcon(getClass().
	    getResource("/images/docsInRed.gif"));
      folderIcon = new ImageIcon(getClass().
	    getResource("/images/folder.gif"));
      redFolderIcon = new ImageIcon(getClass().
	    getResource("/images/folderInRed.gif"));
      unknownIcon = new ImageIcon(getClass().
	    getResource("/images/docsInRed.gif"));
  }


/**
 * Specifies whether or not the directory can be opened.  In order to open the
 * directory, the file must be a directory and the user must have the proper
 * permissions.
 */

  public Boolean isTraversable(File aFile){
    SPFile f = (SPFile) aFile;
    return new Boolean(f.isTraversable());
  }


/**
 * Returns a description of the type of the file, such as "Directory".
 */
  public String getTypeDescription(File aFile) {
    return getFileDescription(aFile);
  }

/**
 * Returns a description of the file.  The description could be something of a
 * short abstract of the file's contents.
 */
  public String getDescription(File aFile) {
    return getFileDescription(aFile);
  }

/**
 * Returns a String representation of the file.
 */

  protected String getFileDescription(File aFile) {
    SPFile f = (SPFile) aFile;
    int type = f.getType();

    String desc = "Unknow Type";

    if (type == SPResourceNode.COLLECTION_TYPE) {
        desc = "Web Folder";
    } else if ( type == SPResourceNode.NON_COLLECTION_TYPE) {
        desc = "Web File";
    }

    return desc;

  }

/**
 * Returns a String representation of the file.
 */
  public String getName(File aFile) {
    if (aFile != null) {
      String name = aFile.getName();
      return name.equals("") ? aFile.getPath() :
                               name;
    }
    return null;
  }

/**
 * Returns a String representation of the file.
 */
  public Color getTextColor(File aFile)  {
    Color textColor = Color.black;  //default color

    SPFile f = (SPFile) aFile;

    boolean isReadOnly = f.getReadOnlyCached();
    boolean noWriteAccess = ! f.hasWriteAccess();

    if (noWriteAccess || isReadOnly) {
        textColor = Color.red;
    }

    return textColor;
  }

/**
 * Returns an icon appropriate for the file.
 */
  public Icon getIcon(File aFile) {
    ImageIcon icon = this.unknownIcon;

    SPFile f = (SPFile) aFile;
    int type = f.getType();
    boolean isReadOnly = f.getReadOnlyCached();
    boolean noWriteAccess = ! f.hasWriteAccess();

    if (noWriteAccess || isReadOnly) {

	if (type == SPResourceNode.COLLECTION_TYPE)  {
	    icon = this.redFolderIcon;
	} else if (type == SPResourceNode.NON_COLLECTION_TYPE) {
	    icon = this.redDocIcon;
	}

    } else {

	if (type == SPResourceNode.COLLECTION_TYPE)  {
            icon = this.folderIcon;
	} else if (type == SPResourceNode.NON_COLLECTION_TYPE) {
            icon = this.docIcon;
	}
    }

    return icon;
  }//getIcon()

} //End of SPFileView class
















