/*
 * $Header: /home/cvs/jakarta-slide/src/share/org/apache/slide/macro/Macro.java,v 1.5 2000/11/25 01:34:53 remm Exp $
 * $Revision: 1.5 $
 * $Date: 2000/11/25 01:34:53 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.slide.macro;

import java.util.Enumeration;
import java.util.Vector;
import java.util.Date;
import org.apache.slide.common.*;
import org.apache.slide.structure.*;
import org.apache.slide.security.*;
import org.apache.slide.content.*;
import org.apache.slide.authenticate.CredentialsToken;

/**
 * Macro helper class.
 * 
 * @author <a href="mailto:remm@apache.org">Remy Maucherat</a>
 * @version $Revision: 1.5 $
 */
public interface Macro {
    
    
    // -------------------------------------------------------------- Constants
    
    
    /**
     * Default macro parameters.
     */
    public static final MacroParameters DEFAULT_PARAMETERS = 
        new MacroParameters();
    
    
    /**
     * Recursive, overwrite macro parameters.
     */
    public static final MacroParameters RECURSIVE_OVERWRITE_PARAMETERS = 
        new MacroParameters(true, true);
    
    
    // ------------------------------------------------------ Interface Methods
    
    
    /**
     * Recursive copy with overwrite macro.
     * 
     * @param token Credentials token
     * @param sourceUri Uri of the source
     * @param destinationUri Uri of the destination
     * @exception CopyMacroException Generic Slide exception
     */
    void copy(SlideToken token, String sourceUri, 
              String destinationUri)
        throws CopyMacroException, DeleteMacroException;
    
    
    /**
     * Copy macro.
     * 
     * @param token Credentials token
     * @param sourceUri Uri of the source
     * @param destinationUri Uri of the destination
     * @param parameters Macro parameters
     * @exception CopyMacroException Generic Slide exception
     */
    void copy(SlideToken token, String sourceUri, 
              String destinationUri, MacroParameters parameters)
        throws CopyMacroException, DeleteMacroException;
    
    
    /**
     * Recursive move with overwrite macro.
     * 
     * @param token Credentials token
     * @param sourceUri Uri of the source
     * @param destinationUri Uri of the destination
     * @exception CopyMacroException Exception occured during copy
     * @exception DeleteMacroException Exception occured during deletion
     */
    void move(SlideToken token, String sourceUri, 
              String destinationUri)
        throws CopyMacroException, DeleteMacroException;
    
    
    /**
     * Move macro.
     * 
     * @param token Credentials token
     * @param sourceUri Uri of the source
     * @param destinationUri Uri of the destination
     * @param parameters Macro parameters
     * @exception CopyMacroException Exception occured during copy
     * @exception DeleteMacroException Exception occured during deletion
     */
    void move(SlideToken token, String sourceUri, 
              String destinationUri, MacroParameters parameters)
        throws CopyMacroException, DeleteMacroException;
    
    
    /**
     * Recursive delete.
     * 
     * @param token Credentials token
     * @param targetUri Uri of the object to delete
     * @exception DeleteMacroException Generic Slide exception
     */
    void delete(SlideToken token, String targetUri)
        throws DeleteMacroException;
    
    
    /**
     * Delete macro.
     * 
     * @param token Credentials token
     * @param targetUri Uri of the source
     * @param parameters Macro parameters, not used right now, 
     * so it can be null
     * @exception DeleteMacroException Generic Slide exception
     */
    void delete(SlideToken token, String targetUri, 
                MacroParameters parameters)
        throws DeleteMacroException;
    
}
