/*
 * $Header: /home/cvs/jakarta-slide/src/webdav/client/src/org/apache/webdav/lib/methods/MkcolMethod.java,v 1.5 2001/05/01 21:28:02 remm Exp $
 * $Revision: 1.5 $
 * $Date: 2001/05/01 21:28:02 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */
 
package org.apache.webdav.lib.methods;

import java.io.InputStream;
import java.io.IOException;

import org.apache.commons.httpclient.HttpMethodBase;

/**
 * The MKCOL method is used to create a new collection. All DAV compliant
 * resources must support the MKCOL method.  Collections are merely
 * the HTTP name for structures like directories or folders (and, in
 * fact, often map directly to a folder or directory on the web server.
 *
 * <p>   This implementation of a MKCOL client method does not support a
 * a request body, and the newly created web collection should therefore
 * have no members.
 *
 * <p>   MKCOL creates a new collection resource at the location specified by
 * the Request-URI. If the resource identified by the Request-URI already
 * exists on the server then the MKCOL will fail.  During MKCOL processing,
 * a server will make the Request-URI a member of the URI's parent collection
 * (unless the Request-URI is "/").  If no parent collection exists, the method
 * will fail.  Thus, for example, if a request to create collection
 * <code>/a/b/c/d/</code> is made, and neither <code>/a/b/</code> nor
 * <code>/a/b/c/</code> exists, the request will fail.
 *
 * <p>   MKCOL is not idempotent (that is to say, each MKCOL request should
 * be handled by the web server, and the results of a MKCOL request should
 * not be cached).
 *
 * <h3>Example Request</h3>
 * <pre>
 * MKCOL /webdisc/xfiles/ HTTP/1.1
 * Host: www.server.org
 * </pre>
 *
 * <h3>Example Response</h3>
 * <pre>
 * HTTP/1.1 201 Created
 * </pre>
 *
 * @author <a href="mailto:remm@apache.org">Remy Maucherat</a>
 */
public class MkcolMethod
    extends HttpMethodBase {
    
    
    // ----------------------------------------------------------- Constructors
    
    
    /**
     * Method constructor.
     */
    public MkcolMethod() {
        name = "MKCOL";
    }
    
    
    /**
     * Method constructor.
     */
    public MkcolMethod(String path) {
        super(path);
        name = "MKCOL";
    }
    
    
    // --------------------------------------------------- WebdavMethod Methods
    
    
    /**
     * Parse the response body.  The MKCOL method does not receive a response
     * body.
     *
     * @param is Input stream
     */
    public void parseResponse(InputStream is)
        throws IOException {
    }


}
