/*
 * $Header: /home/cvs/jakarta-slide/src/webdav/client/src/org/apache/webdav/lib/properties/LockDiscoveryProperty.java,v 1.9 2001/07/30 20:00:38 dirkv Exp $
 * $Revision: 1.9 $
 * $Date: 2001/07/30 20:00:38 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */
package org.apache.webdav.lib.properties;

import java.util.Enumeration;
import java.util.ArrayList;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;

import org.apache.util.DOMUtils;

import org.apache.webdav.lib.ResponseEntity;
import org.apache.webdav.lib.BaseProperty;
import org.apache.webdav.lib.Lock;
import org.apache.webdav.lib.methods.DepthSupport;

/**
 * This class represents a listing of who has lock, what type of lock he has,
 * the timeout type and the time remaining on the timeout, and the associated
 * lock token.  The server is free to withhold any or all of this information
 * if the requesting principal does not have sufficient access rights to see
 * the requested data.
 *
 * <!ELEMENT lockdiscovery (activelock)* >
 *
 * @author Jojada J. Tirtowidjojo at SpeedLegal Holdings Inc.
 * @author <a href="mailto:jericho@thinkfree.com">Park, Sung-Gu</a>
 * @author Remy Maucherat
 * @author Dirk Verbeeck
 * @version $Revision: 1.9 $
 */
public class LockDiscoveryProperty extends BaseProperty {


    // -------------------------------------------------------------- Constants


    /**
     * The property name.
     */
    public static final String TAG_NAME = "lockdiscovery";


    // ----------------------------------------------------------- Constructors


    /**
     * Default constructor for the property.
     */
    public LockDiscoveryProperty(ResponseEntity response, Element element) {
        super(response, element);
    }


    // --------------------------------------------------------- Public Methods


    /** 
     * Get the activelock in this lockdiscovery property.
     * 
     * @return Lock[] A lock array.
     */
    public Lock[] getActiveLocks() {
        NodeList children = element.getChildNodes();
        if (children == null || children.getLength() == 0)
            return null;
        ArrayList locks = new ArrayList();
        for (int i = 0; i < children.getLength(); i++) {
            try {
                Element child = (Element) children.item(i);
                String namespace = DOMUtils.getElementNamespaceURI(child);
                if (namespace != null && namespace.equals("DAV:")) {
                    String localName = DOMUtils.getElementLocalName(child);
                    if ("activelock".equals(localName)) {
                        locks.add(parseLock(child));
                    }
                }
            } catch (ClassCastException e) {
            }
        }
        return (Lock[]) locks.toArray(new Lock[locks.size()]);
    }


    // ------------------------------------------------------ Protected Methods


    /**
     * Parse a lock.
     */
    protected Lock parseLock(Element element) {

        int ls = -1;
        Element child = DOMUtils.getFirstElement(element, "DAV:", "lockscope");
        if (child != null) {
            Element lockScope =
                DOMUtils.getFirstElement(child, "DAV:", "exclusive");
            if (lockScope != null) {
                ls = Lock.SCOPE_EXCLUSIVE;
            }
            lockScope = DOMUtils.getFirstElement(child, "DAV:", "shared");
            if (lockScope != null) {
                ls = Lock.SCOPE_SHARED;
            }
        }
        
        int lt = -1;
        child = DOMUtils.getFirstElement(element, "DAV:", "locktype");
        if (child != null) {
            Element lockType =
                DOMUtils.getFirstElement(child, "DAV:", "write");
            if (lockType != null) {
                lt = Lock.TYPE_WRITE;
            }
        }

        int d = -1;
        child = DOMUtils.getFirstElement(element, "DAV:", "depth");
        if (child != null) {
            String depth = DOMUtils.getTextValue(child);
            if (depth != null) {
                if ("0".equals(depth)) {
                    d = DepthSupport.DEPTH_0;
                } else if ("1".equals(depth)) {
                    d = DepthSupport.DEPTH_1;
                } else if ("infinity".equalsIgnoreCase(depth)) {
                    d = DepthSupport.DEPTH_INFINITY;
                } else {
                    try {
                        d = Integer.parseInt(depth);
                        if (d<0) {
                            d = -1; // unknown
                        }
                    } catch (NumberFormatException ex) {
                        d = -1; // unknown
                    }
                }
            }
        }

        String owner = null;
        child = DOMUtils.getFirstElement(element, "DAV:", "owner");
        owner = DOMUtils.getTextValue(child);

        long t = -1;
        child = DOMUtils.getFirstElement(element, "DAV:", "timeout");
        if (child != null) {
            String timeout = DOMUtils.getTextValue(child);
            int at = timeout.indexOf('-');
            if (at > 0) {
                try {
                    t = Long.parseLong(timeout.substring(at + 1));
                } catch (NumberFormatException e) {
                }
            }
        }

        String lockToken = null;
        child = DOMUtils.getFirstElement(element, "DAV:", "locktoken");
        if (child != null) {
            Element href = DOMUtils.getFirstElement(child, "DAV:", "href");
            if (href != null) {
                lockToken = DOMUtils.getTextValue(href);
            }
        }

        return new Lock(ls, lt, d, owner, t, lockToken);

    }


}

